<?php
namespace Controller;

use Exception\CustomException;
use Zeedhi\Framework\Controller\Simple;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Util\Utilities;
use Util\Messages;
use Util\InstanceProvider;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Call extends Simple
{

    /** @var \Service\Call $callService */
    protected $callService;

    /** @var \Zeedhi\Framework\HTTP\Kernel $kernel */
    protected $kernel;

    public function __construct(\Service\Call $callService, \Zeedhi\Framework\HTTP\Kernel $kernel)
    {
        $this->callService = $callService;
        $this->kernel = $kernel;
    }

    public function countCallsByType(Request $request, Response $response)
    {
        $countCalls = $this->callService->countCallsByType();
        $response->addDataSet(new DataSet('countCallsByType', $countCalls));
    }

    public function countCallsByLabel(Request $request, Response $response)
    {
        $countCalls = $this->callService->countCallsByLabel();
        $response->addDataSet(new DataSet('countCallsByLabel', $countCalls));
    }

    public function callGraphicForFourMonth(Request $request, Response $response)
    {
        try{
            $countCalls = $this->callService->getCallGraphicForFourMonth();
            $response->addDataSet(new DataSet('callGraphicForFourMonth', $countCalls));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function getLastQuestion(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $taskId = \Zeedhi\DevOps\Util\Utilities::getValueInFilterDataSourceByName($conditions,'TASK_ID');
            $countCalls = $this->callService->getLastQuestion($taskId);
            $response->addDataSet(new DataSet('getLastQuestion', $countCalls));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findMenuItems(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $productId = \Zeedhi\DevOps\Util\Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCT_ID');
            $menus = $this->callService->findMenuItems($productId);
            \Zeedhi\DevOps\Util\Utilities::setDataSetDefault($request,$response,$menus);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findDatasetDashboardCardSLA(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();

            $menus = $this->callService->findDatasetDashboardCardSLA($productId);
            \Zeedhi\DevOps\Util\Utilities::setDataSetDefault($request,$response,$menus);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findDashboardPizza(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            List($periodSlaStart,$periodSlaEnd) = Utilities::getValueInFilterDataSourceByName($conditions,'PERIOD_SLA');

            $dataset = $this->callService->findDashboardPizza($periodSlaStart,$periodSlaEnd);
            foreach ($dataset as &$item) {
                $item['TOTAL'] = (int)$item['TOTAL'];
                $item['ID_CLA'] = DevOpsUtilities::setSelectMultiInQuery($item['ID_CLA'],false, ',',true );
            }
            \Zeedhi\DevOps\Util\Utilities::setDataSetDefault($request,$response,$dataset);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findSLAClassification(Request\Filter $request, Response $response)
    {
        try{
            $dataset = $this->callService->findSLAClassification();
            \Zeedhi\DevOps\Util\Utilities::setDataSetDefault($request,$response,$dataset);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findDashboardBarOpenAndBacklogByProduct(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();

            List($periodSlaStart,$periodSlaEnd) = Utilities::getValueInFilterDataSourceByName($conditions,'PERIOD_SLA');
            $dataset = $this->callService->findDashboardBarOpenAndBacklogByProduct($periodSlaStart,$periodSlaEnd);
            foreach ($dataset as &$item) {
                $item['TOTAL_BACK'] = (int)$item['TOTAL_BACK'];
                $item['TOTAL_BACK_SLA'] = (int)$item['TOTAL_BACK_SLA'];
                $item['TOTAL_OPEN'] = (int)$item['TOTAL_OPEN'];
                $item['TOTAL_OPEN_SLA'] = (int)$item['TOTAL_OPEN_SLA'];
            }

            \Zeedhi\DevOps\Util\Utilities::setDataSetDefault($request,$response,$dataset);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function gridSlaDashboard(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            List($periodSlaStart,$periodSlaEnd) = Utilities::getValueInFilterDataSourceByName($conditions,'PERIOD_SLA');
            $dataset = $this->callService->gridSlaDashboard($periodSlaStart,$periodSlaEnd);
            if(!empty($dataset)){
                $row = $dataset[0];
                $dataset[] = array(
                    'NAME'=> 'Chamados SLA',
                    'OPENED'=> $row['OPENED_SLA'],
                    'COMPETED'=> $row['COMPETED_SLA'],
                    'BACKLOG'=> $row['BACKLOG_SLA']
                );
                $dataset[] = array(
                    'NAME'=> 'Fora SLA',
                    'OPENED'=> $row['OPENED_SLA_OUT'],
                    'COMPETED'=> $row['COMPETED_SLA_OUT'],
                    'BACKLOG'=> $row['BACKLOG_SLA_OUT']
                );
                $dataset[] = array(
                    'NAME'=> '% SLA',
                    'OPENED' => Utilities::percent($row['OPENED_SLA_OUT'],$row['OPENED_SLA']) . '%',
                    'COMPETED' => Utilities::percent($row['COMPETED_SLA_OUT'],$row['COMPETED_SLA']) . '%',
                    'BACKLOG' =>Utilities::percent($row['BACKLOG_SLA_OUT'],$row['BACKLOG_SLA']) . '%',
                );
            }
            \Zeedhi\DevOps\Util\Utilities::setDataSetDefault($request,$response,$dataset);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function callBudget(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $nrsoliserv = \Zeedhi\DevOps\Util\Utilities::getValueInFilterDataSourceByName($conditions,'NRSOLISERV');
            $nameProposal = \Zeedhi\DevOps\Util\Utilities::getValueInFilterDataSourceByName($conditions,'NAME_PROPOSAL');
            $nameDiscount = \Zeedhi\DevOps\Util\Utilities::getValueInFilterDataSourceByName($conditions,'NAME_DISCOUNT');
            $lang = \Zeedhi\DevOps\Util\Utilities::getValueInFilterDataSourceByName($conditions,'LANG');
            $menus = $this->callService->findCallBudget($nrsoliserv,$nameProposal,$nameDiscount,$lang);
            \Zeedhi\DevOps\Util\Utilities::setDataSetDefault($request,$response,$menus);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function openPageRequisite(Request\DataSet $request, Response $response)
    {
        try{
            $nrsoliserv = $request->getDataSet()->getRows()[0]['ID'];
            $this->callService->getPageRequisite($nrsoliserv);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findProductVersion(Request\Filter $request, Response $response)
    {
        try{
            $conditions = $request->getFilterCriteria()->getConditions();
            $productId = \Zeedhi\DevOps\Util\Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCT_ID',true);
            $page = $request->getFilterCriteria()->getPage();
            $pageSize = $request->getFilterCriteria()->getPageSize();
            $menus = $this->callService->findProductVersion($productId,$pageSize,$page,$conditions);
            \Zeedhi\DevOps\Util\Utilities::setDataSetDefault($request,$response,$menus);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findCalls(Request $request, Response $response)
    {
        ini_set('max_execution_time', '900');
        set_time_limit(900);
        try{
            /** @var $request Request\Filter*/
            $conditions     = $request->getFilterCriteria()->getConditions();
            $page           = $request->getFilterCriteria()->getPage();
            $pageSize       = $request->getFilterCriteria()->getPageSize();
            $orderBy        = $request->getFilterCriteria()->getOrderBy();
            $cdcliente      = Utilities::getValueInFilterDataSourceByName($conditions,'CDCLIENTE', true);
            $groupByLabel   = Utilities::getValueInFilterDataSourceByName($conditions,'GROUP_BY_LABEL', true);
            $mainType       = Utilities::getValueInFilterDataSourceByName($conditions,'MAIN_TYPE_CALL', true);
            $SLA            = Utilities::getValueInFilterDataSourceByName($conditions,'SLA', true);
            $SLAclas        = Utilities::getValueInFilterDataSourceByName($conditions,'SLA_CLA', true);
            $cdProd        = Utilities::getValueInFilterDataSourceByName($conditions,'PRODUCT_FILTER_ID', true);
            foreach($conditions as $i => $condition){
                if($condition['columnName'] == "REFERENCE"){
                    $conditions[$i]["value"] = str_replace("'","",$condition["value"]);
                }
            }
            $mainType = 'P';
            $calls          = $this->callService->findCalls($mainType, $conditions, $orderBy, $page, $pageSize,
                $cdcliente,$groupByLabel,$SLA,$SLAclas, $cdProd);
            $response->addDataSet(new DataSet('findCalls', $calls));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function saveSupportCall(Request\Dataset $request, Response $response) {
        try{
            $supportCall = $request->getDataSet()->getRows()[0];
            $id          = empty($supportCall['ID']) ? null : $supportCall['ID'];
            $reference   = empty($supportCall['REFERENCE']) ? null : $supportCall['REFERENCE'];
            $product     = empty($supportCall['PRODUCT']) ? null : $supportCall['PRODUCT'];
            $cause       = empty($supportCall['CAUSE']) ? null : $supportCall['CAUSE'];
            $description = empty($supportCall['DESCRIPTION']) ? null : $supportCall['DESCRIPTION'];
            $reportedBy  = empty($supportCall['REPORTER_ID']) ? $supportCall['REPORTED_BY'] : $supportCall['REPORTER_ID'];
            if(empty($reportedBy)){
                $reportedBy = empty($productCall['REPORTER_BY']) ?  null : $productCall['REPORTER_BY'];
            }
            $callLabels  = empty($supportCall['CALL_LABELS']) ? array() : $supportCall['CALL_LABELS'];
            $callRelationship = empty($supportCall['CALL_RELATIONSHIP']) ? null : $supportCall['CALL_RELATIONSHIP'];
            if((!empty($supportCall['__is_new']) && $supportCall['__is_new']) || (!$id && $callRelationship)){
                $newSupportCall = $this->callService->saveNewSupportCall($reference, $description, $product, $cause, $reportedBy, $callLabels, $callRelationship);
                $supportCall['ID'] = $newSupportCall['ID'];
            } else {
                $this->callService->updateSupportCall($id, $reference, $description, $product, $cause,  $reportedBy, $callLabels);
            }
            $response->addDataSet(new DataSet('supportCall', array($supportCall)));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function declinePreBudgetCall(Request\DataSet $request, Response $response){
        $call = $request->getDataSet()->getRows()[0];
        $callRelationship = $call['CALL_RELATIONSHIP'];
        $description = $call['DESCRIPTION'];
        $this->callService->declinePreBudget($callRelationship,$description);
    }

    public function approvePreBudget(Request\DataSet $request, Response $response){
        $call = $request->getDataSet()->getRows()[0];
        $callRelationship = $call['CALL_RELATIONSHIP'];
        $description = $call['DESCRIPTION'];
        $this->callService->approvePreBudget($callRelationship,$description);
    }

    public function getInvisibleFields($call)
    {
        $invisibleFields = array();
        // Invisible fields //
        $invisibleFields["TEXT_SMALL_1"] = empty($call['TEXT_SMALL_1']) ? null : $call['TEXT_SMALL_1'];
        $invisibleFields["TEXT_SMALL_2"] = empty($call['TEXT_SMALL_2']) ? null : $call['TEXT_SMALL_2'];
        $invisibleFields["TEXT_SMALL_3"] = empty($call['TEXT_SMALL_3']) ? null : $call['TEXT_SMALL_3'];
        $invisibleFields["TEXT_SMALL_4"] = empty($call['TEXT_SMALL_4']) ? null : $call['TEXT_SMALL_4'];
        $invisibleFields["TEXT_SMALL_5"] = empty($call['TEXT_SMALL_5']) ? null : $call['TEXT_SMALL_5'];
        $invisibleFields["TEXT_LONG_1"] = empty($call['TEXT_LONG_1']) ? null : $call['TEXT_LONG_1'];
        $invisibleFields["TEXT_LONG_2"] = empty($call['TEXT_LONG_2']) ? null : $call['TEXT_LONG_2'];
        $invisibleFields["TEXT_LONG_3"] = empty($call['TEXT_LONG_3']) ? null : $call['TEXT_LONG_3'];
        $invisibleFields["TEXT_LONG_4"] = empty($call['TEXT_LONG_4']) ? null : $call['TEXT_LONG_4'];
        $invisibleFields["TEXT_LONG_5"] = empty($call['TEXT_LONG_5']) ? null : $call['TEXT_LONG_5'];
        $invisibleFields["DATE_1"] = empty($call['DATE_1']) ? null : $call['DATE_1'];
        $invisibleFields["DATE_2"] = empty($call['DATE_2']) ? null : $call['DATE_2'];
        $invisibleFields["DATE_3"] = empty($call['DATE_3']) ? null : $call['DATE_3'];
        $invisibleFields["DATE_4"] = empty($call['DATE_4']) ? null : $call['DATE_4'];
        $invisibleFields["DATE_5"] = empty($call['DATE_5']) ? null : $call['DATE_5'];
        $invisibleFields["NUMBER_1"] = empty($call['NUMBER_1']) ? null : $call['NUMBER_1'];
        $invisibleFields["NUMBER_2"] = empty($call['NUMBER_2']) ? null : $call['NUMBER_2'];
        $invisibleFields["NUMBER_3"] = empty($call['NUMBER_3']) ? null : $call['NUMBER_3'];
        $invisibleFields["NUMBER_4"] = empty($call['NUMBER_4']) ? null : $call['NUMBER_4'];
        $invisibleFields["NUMBER_5"] = empty($call['NUMBER_5']) ? null : $call['NUMBER_5'];

        return $invisibleFields;
    }
    public function saveCall(Request\DataSet $request, Response $response)
    {
        try {
            $call           = $request->getDataSet()->getRows()[0];
            $mainTypeCall   = empty($call['MAIN_TYPE_CALL']) ? null : $call['MAIN_TYPE_CALL'];
            $id             = empty($call['ID']) ? null : $call['ID'];
            $callType       = empty($call['TYPE']) ? null : $call['TYPE'];
            $reference      = empty($call['REFERENCE']) ? null : $call['REFERENCE'];
            $product        = empty($call['PRODUCT']) ? null : $call['PRODUCT'];
            $cause          = empty($call['CAUSE']) ? null : $call['CAUSE'];
            $board          = empty($call['BOARD']) ? null : $call['BOARD'];
            $classification = empty($call['CLASSIFICATION']) ? null : $call['CLASSIFICATION'];
            $description    = empty($call['DESCRIPTION']) ? null : $call['DESCRIPTION'];
            $callLabels     = empty($call['CALL_LABELS']) ? array() : $call['CALL_LABELS'];
            $productType    = empty($call['PRODUCT_TYPE']) ? null : $call['PRODUCT_TYPE'];
            $productVersion = isset($call['PRODUCT_VERSION']) ? $call['PRODUCT_VERSION'] : null;
            $productRelease = isset($call['PRODUCT_RELEASE']) ? $call['PRODUCT_RELEASE'] : null;
            $productPatch   = isset($call['PRODUCT_PATCH']) ? $call['PRODUCT_PATCH'] : null;
            $useLicense     = empty($call['USE_LICENSE']) ? null : $call['USE_LICENSE'];
            $reportedBy     = empty($call['REPORTED_BY']) ?  null : $call['REPORTED_BY'];
            $callRelationship = empty($call['CALL_RELATIONSHIP']) ? null : $call['CALL_RELATIONSHIP'];
            $devopsId = empty($call['DEVOPS_ID']) ? null : $call['DEVOPS_ID'];
            $menuItemId = empty($call['MENU_ITEM_ID']) ? null : $call['MENU_ITEM_ID'];
            $menuVersionId = empty($call['MENU_VERSION_ID']) ? null : $call['MENU_VERSION_ID'];
            $invisibleFields = $this->getInvisibleFields($call);
            if(empty($reportedBy)){
                $reportedBy = empty($productCall['REPORTER_BY']) ?  null : $productCall['REPORTER_BY'];
            }
            if($mainTypeCall == 'P'){
                if((!empty($call['__is_new']) && $call['__is_new']) || (!$id && $callRelationship)){
                    $newProductCall = $this->callService->saveNewProductCall($callType, $reference, $product,
                        $productType, $productVersion, $useLicense, $classification, $board, $description,
                        $reportedBy, $callLabels, $productRelease, $productPatch, $callRelationship,$devopsId,
                        $menuItemId,$menuVersionId, null,$invisibleFields);
                    $call['ID'] = $newProductCall['ID'];
                } else {
                    $this->callService->updateProductCall($callType, $id,  $reference, $product,
                        $productType, $productVersion, $useLicense, $classification, $board,
                        $description, $reportedBy, $callLabels,$devopsId,
                        $menuItemId,$menuVersionId, $invisibleFields);

                }
            } else if ($mainTypeCall == 'S'){
                if((!empty($supportCall['__is_new']) && $supportCall['__is_new']) || (!$id && $callRelationship)){
                    $newSupportCall = $this->callService->saveNewTicketTypeCall($reference, $product, $description, $callLabels,
                        $callRelationship, $reportedBy,$classification, $invisibleFields);
                    $call['ID'] = $newSupportCall['ID'];
                } else {
                    $this->callService->updateSupportCall($id, $reference, $description, $product, $cause,  $reportedBy, $callLabels);
                }
            }
            $response->addDataSet(new DataSet('call', array($call)));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function saveProductCall(Request\DataSet $request, Response $response)
    {
        ini_set('max_execution_time', '60');
        set_time_limit(60);
        try {
            $productCall = $request->getDataSet()->getRows()[0];
            $id             = empty($productCall['ID']) ? null : $productCall['ID'];
            $callType       = empty($productCall['TYPE']) ? null : $productCall['TYPE'];
            $reference      = empty($productCall['REFERENCE']) ? null : $productCall['REFERENCE'];
            $product        = empty($productCall['PRODUCT']) ? null : $productCall['PRODUCT'];
            $projectId      = empty($productCall['PROJECT_ID']) ? null : $productCall['PROJECT_ID'];
            $board          = empty($productCall['BOARD']) ? null : $productCall['BOARD'];
            $classification = empty($productCall['CLASSIFICATION']) ? null : $productCall['CLASSIFICATION'];
            $description    = empty($productCall['DESCRIPTION']) ? null : $productCall['DESCRIPTION'];
            $callLabels     = empty($productCall['CALL_LABELS']) ? array() : $productCall['CALL_LABELS'];
            $productType    = empty($productCall['PRODUCT_TYPE']) ? null : $productCall['PRODUCT_TYPE'];
            $productVersion = isset($productCall['PRODUCT_VERSION']) ? $productCall['PRODUCT_VERSION'] : null;
            $productRelease = isset($productCall['PRODUCT_RELEASE']) ? $productCall['PRODUCT_RELEASE'] : null;
            $productPatch   = isset($productCall['PRODUCT_PATCH']) ? $productCall['PRODUCT_PATCH'] : null;
            $useLicense     = empty($productCall['USE_LICENSE']) ? null : $productCall['USE_LICENSE'];
            $reportedBy     = empty($productCall['REPORTED_BY']) ?  null : $productCall['REPORTED_BY'];
            $cdcliente      = empty($productCall['CDCLIENTE']) ?  null : $productCall['CDCLIENTE'];
            $devopsId = empty($productCall['DEVOPS_ID']) ? null : $productCall['DEVOPS_ID'];
            $menuItemId = empty($productCall['MENU_ITEM_ID']) ? null : $productCall['MENU_ITEM_ID'];
            $menuVersionId = empty($productCall['MENU_VERSION_ID']) ? null : $productCall['MENU_VERSION_ID'];
            $invisibleFields = $this->getInvisibleFields($productCall);
            if(empty($product)){
                $product = $projectId;
            }
            if(empty($reportedBy)){
                $reportedBy = empty($productCall['REPORTER_BY']) ?  null : $productCall['REPORTER_BY'];
            }
            $callRelationship = empty($productCall['CALL_RELATIONSHIP']) ? null : $productCall['CALL_RELATIONSHIP'];
            if(empty($classification)){
                $classification = empty($productCall['CDMOTICONT']) ? null : $productCall['CDMOTICONT'];
            }
            if((!empty($productCall['__is_new']) && $productCall['__is_new']) || (!$id && $callRelationship)){
                $newProductCall = $this->callService->saveNewProductCall($callType, $reference, $product, $productType,
                    $productVersion, $useLicense, $classification, $board, $description, $reportedBy,
                    $callLabels, $productRelease, $productPatch, $callRelationship,$devopsId,$menuItemId,$menuVersionId,$cdcliente, $invisibleFields);
                $productCall['ID'] = $newProductCall['ID'];
                $productCall['CALL_RELATIONSHIP'] = $newProductCall['CALL_RELATIONSHIP'];
            } else {
                $this->callService->updateProductCall($callType, $id,  $reference, $product, $productType, $productVersion, $useLicense, $classification, $board, $description, $reportedBy, $callLabels, null, null, null, $invisibleFields);
            }
            $response->addDataSet(new DataSet('productCall', array($productCall)));
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function findMyMostRecentlyUpdatedCalls(Request\Filter $request, Response $response)
    {
        $params = $request->getFilterCriteria()->getConditions();
        if(!empty($params[0])) {
            try{
                $numberOfProductCalls = $params[0]['value'];
                $calls = $this->callService->findMyMostRecentlyUpdatedCalls($numberOfProductCalls);
                $response->addDataSet(new DataSet("findMyMostRecentlyUpdatedCalls", $calls));
            } catch(\Exception $e) {
                $returnMessage = $e->getMessage();
                $response->addNotification(new Response\Notification($returnMessage, Response\Notification::TYPE_ALERT));
            }
        }
    }

    public function getInfoForSendCall(Request\Filter $request, Response $response)
    {
        $conditions   = $request->getFilterCriteria()->getConditions();
        $callId       = Utilities::getValueInFilterDataSourceByName($conditions,'ID');
        $lang         = Utilities::getValueInFilterDataSourceByName($conditions,'LANG');
        $productCalls = $this->callService->getInfoForSendCall($callId, $lang);
        $response->addDataSet(new DataSet('getInfoForSendCall', $productCalls));
    }

    public function operadorByResponse(Request\Filter $request, Response $response)
    {
        $conditions     = $request->getFilterCriteria()->getConditions();
        $callId         = Utilities::getValueInFilterDataSourceByName($conditions,'ID');
        $callRelationship = Utilities::getValueInFilterDataSourceByName($conditions,'CALL_RELATIONSHIP');
        $cdfastramdes   = Utilities::getValueInFilterDataSourceByName($conditions,'CDFASTRAMDES');
        $nrfastramdes   = Utilities::getValueInFilterDataSourceByName($conditions,'NRFASTRAMDES');
        $callType       = Utilities::getValueInFilterDataSourceByName($conditions,'CALL_TYPE');
        $productCalls   = $this->callService->operadorByResponse($callId, $callType, $cdfastramdes, $nrfastramdes, $callRelationship);
        $response->addDataSet(new DataSet('operadorByResponse', $productCalls));
    }

    public function sendCallGep(Request\DataSet $request, Response $response)
    {
        try {

            $row                = $request->getDataSet()->getRows()[0];
            $cdfastramdes       = $row['CDFASTRAMDES'];
            $cdoperador         = $row['CDOPERADOR'];
            $comment            = $row['COMMENT'];
            $nrfastramdes       = $row['NRFASTRAMDES'];
            $txacaofatram       = $row['TXACAOFATRAM'];
            $callRelationship   = $row['CALL_RELATIONSHIP'];
            $document           = empty($row['DOCUMENT']) ?  array() : $row['DOCUMENT'];
            $documentFileServer           = empty($row['DOCUMENTS_FILE_SERVER']) ?  array() : $row['DOCUMENTS_FILE_SERVER'];
            $callType           = $row['CALL_TYPE'];
            $this->callService->sendCallGep(null, $cdfastramdes, $cdoperador, $comment, $nrfastramdes, $txacaofatram,
                $callType, $callRelationship, $document, $documentFileServer);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }


    public function findProductCallProducts(Request\Filter $request, Response $response)
    {
        $params = $request->getFilterCriteria()->getConditions();
        try {
            $httpRequest    = $this->kernel->getHttpRequest();
            $filter         = $httpRequest->get('filter');
            $itemsPerPage   = $httpRequest->get('itemsPerPage');
            $page           = $httpRequest->get('page');

            $productCallProducts = array();
            $applicationType = DevOpsUtilities::getValueInFilterDataSourceByName($params, 'APPLICATION_TYPE', true);
            $productType = DevOpsUtilities::getValueInFilterDataSourceByName($params, 'PRODUCT_TYPE', true);

            if($applicationType === \Service\Call::TYPE_ISSUE) {
                $productCallProducts = InstanceProvider::getService()->getPmIntegration()->findProductCallProducts($params, $itemsPerPage, $page);
            } else if ($applicationType === \Service\Call::TYPE_RICH_CLIENT) {
                if($productType) {
                    $productCallProducts = $this->callService->findProductCallProductsFromRichClient($productType, $params);
                }
            } else if ($applicationType === \Service\Call::TYPE_TICKET) {
                $productCallProducts = $this->callService->findProductCallProductsFromTicket($params);
            }
            $response->addDataSet(new DataSet('productCallProducts', $productCallProducts));

        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function findProductCallProductsNew(Request\Filter $request, Response $response)
    {
        $params = $request->getFilterCriteria()->getConditions();
        try {
            $httpRequest    = $this->kernel->getHttpRequest();
            $filter         = $httpRequest->get('filter');
            $itemsPerPage   = $httpRequest->get('itemsPerPage');
            $page           = $httpRequest->get('page');


            $productCallProducts = $this->callService->findProductCallProductsNew();
            $response->addDataSet(new DataSet('productCallProducts', $productCallProducts));

        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function findProductCallClassification(Request\Filter $request, Response $response) {
        $params = $request->getFilterCriteria()->getConditions();
        try {
            $httpRequest  = $this->kernel->getHttpRequest();
            $itemsPerPage = $httpRequest->get('itemsPerPage');
            $page         = $httpRequest->get('page');

            $productCallClassification = array();
            $applicationType = DevOpsUtilities::getValueInFilterDataSourceByName($params, 'APPLICATION_TYPE', true);

            if($applicationType === \Service\Call::TYPE_ISSUE) {
                $productCallClassification = InstanceProvider::getService()->getPmIntegration()->findProductCallClassification($params, $itemsPerPage, $page);
            } else if($applicationType === \Service\Call::TYPE_RICH_CLIENT) {
                $productCallClassification = $this->callService->findProductCallClassificationFromRichClient($params);
            }
            $response->addDataSet(new DataSet('productCallClassification', $productCallClassification));
        } catch(\Exception $e) {
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function closeProductCall(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $callId = $row['ID'];
            $callRelationship = $row['CALL_RELATIONSHIP'];
            $reference = $row['REFERENCE'];
            $product = $row['PRODUCT'];
            $classification = $row['CLASSIFICATION'];
            $board = $row['BOARD'];
            $description = $row['DESCRIPTION'];
            $document = empty($row['DOCUMENT']) ?  array() : $row['DOCUMENT'];
            $comment = $row['COMMENT'];
            $this->callService->closeProductCall($callId, $callRelationship, $reference, $product, $classification,
                $board, $description, $comment, $document);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function reopenIssue(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $callId = $row['ID'];
            $callRelationship = $row['CALL_RELATIONSHIP'];
            $reference = $row['REFERENCE'];
            $product = $row['PRODUCT'];
            $classification = $row['CLASSIFICATION'];
            $board = $row['BOARD'];
            $description = $row['DESCRIPTION'];
            $comment = $row['COMMENT'];
            $document = empty($row['DOCUMENT']) ?  array() : $row['DOCUMENT'];
            $this->callService->reopenIssue($callId, $comment, $callRelationship, $reference, $product, $classification,
                $board, $description,$document);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function refuseIssue(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $callId = $row['ID'];
            $callRelationship = $row['CALL_RELATIONSHIP'];
            $reference = $row['REFERENCE'];
            $product = $row['PRODUCT'];
            $classification = $row['CLASSIFICATION'];
            $board = $row['BOARD'];
            $description = $row['DESCRIPTION'];
            $comment = $row['COMMENT'];
            $document = $row['DOCUMENT'];
            $this->callService->refuseIssue($callId,$comment,$callRelationship,$reference,$product,$classification,
                $board,$description,$document);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function approveIssue(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $callId = $row['ID'];
            $callRelationship = $row['CALL_RELATIONSHIP'];
            $reference = $row['REFERENCE'];
            $product = $row['PRODUCT'];
            $classification = $row['CLASSIFICATION'];
            $board = $row['BOARD'];
            $description = $row['DESCRIPTION'];
            $comment = $row['COMMENT'];
            $document = empty($row["DOCUMENT"]) ? null : $row["DOCUMENT"];
            $this->callService->approveIssue($callId,$comment,$callRelationship,$reference,$product,$classification,
                $board,$description,$document);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function giveUpImplementationIssue(Request\DataSet $request, Response $response)
    {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $callId = $row['ID'];
            $callRelationship = $row['CALL_RELATIONSHIP'];
            $reference = $row['REFERENCE'];
            $product = $row['PRODUCT'];
            $classification = $row['CLASSIFICATION'];
            $board = $row['BOARD'];
            $description = $row['DESCRIPTION'];
            $comment = empty($row["COMMENT"]) ? null : $row['COMMENT'];
            $document = empty($row["DOCUMENT"]) ? null : $row["DOCUMENT"];
            $this->callService->giveUpImplementationIssue($callId,$comment,$callRelationship,$reference,$product,$classification,
                $board,$description,$document);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function findProductCallProductPatch(Request\Filter $request, Response $response)
    {
        $params = $request->getFilterCriteria()->getConditions();

        $productType = Utilities::getValueInFilterDataSourceByName($params,'PRODUCT_TYPE');
        $version = Utilities::getValueInFilterDataSourceByName($params,'PRODUCT_VERSION');
        $release = Utilities::getValueInFilterDataSourceByName($params,'PRODUCT_RELEASE');

        $productCallProductVersions = $this->callService->findProductCalProductPatch($productType, $version, $release);
        $response->addDataSet(new DataSet('findProductCallProductPatch', $productCallProductVersions));

    }

    public function findProductCallProductReleases(Request\Filter $request, Response $response)
    {
        $params = $request->getFilterCriteria()->getConditions();
        $productType = Utilities::getValueInFilterDataSourceByName($params,'PRODUCT_TYPE');
        $version = Utilities::getValueInFilterDataSourceByName($params,'PRODUCT_VERSION');

        $productCallProductVersions = $this->callService->findProductCallProductReleases($productType, $version);
        $response->addDataSet(new DataSet('findProductCallProductReleases', $productCallProductVersions));
    }

    public function findProductCallProductVersions(Request\Filter $request, Response $response)
    {
        $params = $request->getFilterCriteria()->getConditions();
        if(empty($params)) {
            $response->addDataSet(new DataSet('productCallProductVersions', array()));
        } else {
            $productType = $params[0]["value"];
            $product = $params[1]["value"];
            $productCode = $params[2]["value"];


            $productCallProductVersions = $this->callService->findProductCalProductVersions($productType, $product, $productCode);
            $response->addDataSet(new DataSet('productCallProductVersions', $productCallProductVersions));
        }
    }

    public function findProductCallUseLicense(Request\Filter $request, Response $response)
    {
        try {
            $httpRequest = $this->kernel->getHttpRequest();
            $itemsPerPage = $httpRequest->get('itemsPerPage');
            $page = $httpRequest->get('page');

            $conditions = $request->getFilterCriteria()->getConditions();
            $cdCliente = DevOpsUtilities::getValueInFilterDataSourceByName($conditions,'CDCLIENTE',true);
            $filters = DevOpsUtilities::createQueryFiltered($conditions);

            $productCallUserLicenses = $this->callService->findProductCallUseLicense($filters,$cdCliente);

            $productCallUserLicenses = array_slice($productCallUserLicenses, ($page-1)*$itemsPerPage, $itemsPerPage);
            $response->addDataSet(new DataSet('productCallUserLicenses', $productCallUserLicenses));
        } catch(\Exception $e) {
            $response->addDataSet(new DataSet('productCallUserLicenses', array()));
        }
    }


    public function getLinkDownloadBudgetSm(Request\DataSet $request, Response $response)
    {
        try {

            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $issueId = isset($row['ISSUE_ID']) ? $row['ISSUE_ID'] : null;
            $this->callService->getLinkDownloadBudgetSm($issueId);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function getLinkDownloadBudgetSmSoliserv(Request\DataSet $request, Response $response)
    {
        try {

            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            $nrsoliserv = isset($row['NRSOLISERV']) ? $row['NRSOLISERV'] : null;
            $this->callService->getLinkDownloadBudgetSmSoliserv($nrsoliserv);
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function findProductCallProductTypes(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();

            $productCode = DevOpsUtilities::getValueInFilterDataSourceByName($conditions,'PRODUCT_CODE', true);
            $filters = DevOpsUtilities::createQueryFiltered($conditions);
            $productCallProducTypes = $this->callService->findProductCallProductTypes($filters, $productCode);
            $response->addDataSet(new DataSet('productCallProductTypes', $productCallProducTypes));
        } catch(\Exception $e) {
            $response->addDataSet(new DataSet('productCallProductTypes', array()));
        }
    }

    public function findProductCallTypes(Request\Filter $request, Response $response)
    {
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $isNew = DevOpsUtilities::getValueInFilterDataSourceByName($conditions,'IS_NEW');
            $productCallTypes = $this->callService->findProductCallTypes($isNew);
            DevOpsUtilities::setDataSetDefault($request,$response,$productCallTypes);
        } catch(\Exception $e) {
            DevOpsUtilities::setDataSetDefault($request,$response,array());
        }
    }

    public function questionForSendCall(Request\Filter $request, Response $response)
    {
        $conditions         = $request->getFilterCriteria()->getConditions();
        $callId             = Utilities::getValueInFilterDataSourceByName($conditions,'ID');
        $lang               = Utilities::getValueInFilterDataSourceByName($conditions,'LANG');
        $callRelationship   = Utilities::getValueInFilterDataSourceByName($conditions,'CALL_RELATIONSHIP');
        $callType   = Utilities::getValueInFilterDataSourceByName($conditions,'CALL_TYPE');
        $productCalls = $this->callService->getQuestionForSendCall($callId, $lang, $callType, $callRelationship);
        $response->addDataSet(new DataSet('questionForSendCall', $productCalls));
    }

    public function getCall(Request $request, Response $response)
    {
        $conditions         = $request->getFilterCriteria()->getConditions();
        $callType           = Utilities::getValueInFilterDataSourceByName($conditions,'TYPE');
        $callRelationship   = Utilities::getValueInFilterDataSourceByName($conditions,'CALL_RELATIONSHIP');
        $call = $this->callService->getCall($callRelationship, $callType);
        $response->addDataSet(new DataSet('call', array($call)));
    }

    public function findSupportCallCause(Request\Filter $request, Response $response)
    {
        $params = $request->getFilterCriteria()->getConditions();
        $product = empty($params[0]['value']) ? "" : $params[0]['value'];
        $causes = $this->callService->findSupportCallCause($product);
        $response->addDataSet(new DataSet('supportCallsCauses', $causes));
    }

    public function deleteSupportCalls(Request\DataSet $request, Response $response)
    {
        try {
            $supportCalls = $request->getDataSet()->getRows();
            foreach($supportCalls as $supportCall) {
                $supportCallID = $supportCall['ID'];
                $associatedCasoID = $supportCall['CALL_RELATIONSHIP'];
                $this->callService->deleteSupportCall($supportCallID, $associatedCasoID);
            }
        } catch(CustomException $e) {
            $response->setError(Utilities::getErrorObjectFromException($e));
        }
    }

    public function findResultevol(Request\Filter $request, Response $response)
    {
        try{
            $resultevol = $this->callService->findResultevol();
            $response->addDataSet(new DataSet("findResultevol", $resultevol));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }

    }

    public function findSupportCallProducts(Request\Filter $request, Response $response)
    {
        $products = $this->callService->findSupportCallProducts();
        $response->addDataSet(new DataSet('supportCallsProducts', $products));
    }

    public function closeSupportCall(Request\Dataset $request, Response $response)
    {
        try{
            $row                = $request->getDataSet()->getRows()[0];
            $comment            = empty($row['COMMENT']) ? null : $row['COMMENT'];
            $callRelationship   = $row['CALL_RELATIONSHIP'];
            $this->callService->closeSupportCall($callRelationship, $comment);
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function updateSolicitationEndDate(Request $request, Response $response){

        $row = $request->getRow();

        $updateParams = array(
            "NRSOLISERV"    => $row["NRSOLISERV"],
            "CDFILIAL"      => $row["CDFILIAL"],
            "DTCOMPROMISS"  => $row["END_DATE"]
        );

        $this->callService->updateSolicitationEndDate($updateParams);

    }

    public function updateTaskEndDate(Request $request, Response $response){

        $row = $request->getRow();
        $this->callService->updateTaskEndDate($row["ID"], $row["END_DATE"]);

    }

    public function checkTeknisaOperator(Request $request, Response $response){

        $isTeknisaOperator = Utilities::currentUserIsTeknisa();
        $response->addDataSet(new DataSet("data", array("IS_TEKNISA_OPERATOR" => $isTeknisaOperator)));

    }
    public function saveDefaultPreferences(Request\Dataset $request, Response $response) {
        try{
            $row         = $request->getDataSet()->getRows()[0];
            $view        = empty($row['ID_VIEW']) ? null : $row['ID_VIEW'];
            $filter      = empty($row['ID_FILTER']) ? null : $row['ID_FILTER'];
            $layout      = empty($row['ID_LAYOUT']) ? null : $row['ID_LAYOUT'];

            $response = $this->callService->saveDefaultPreferences($view, $filter, $layout);

        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findDefaultPreferences(Request $request, Response $response) {
        try{
            $preferences = $this->callService->findDefaultPreferences();
            $response->addDataSet(new DataSet('findDefaultPreferences', $preferences));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function findPreferencesPerId(Request $request, Response $response) {
        try{
            $conditions         = $request->getFilterCriteria()->getConditions();
            $id           = Utilities::getValueInFilterDataSourceByName($conditions,'ID');
            $type          = Utilities::getValueInFilterDataSourceByName($conditions,'TYPE');
            $preference = $this->callService->findPreferencesPerId($id, $type);
            $response->addDataSet(new DataSet('findPreferencesPerId', $preference));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function slaDetails(Request $request, Response $response) {
        try{
            $conditions         = $request->getFilterCriteria()->getConditions();
            $cdfilial          = Utilities::getValueInFilterDataSourceByName($conditions,'CDFILIAL');
            $callRelationship      = Utilities::getValueInFilterDataSourceByName($conditions,'CALL_RELATIONSHIP');
            $result = $this->callService->slaDetails($cdfilial, $callRelationship);
            $response->addDataSet(new DataSet('slaDetails', $result));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }

    public function verifySlaVisibility(Request $request, Response $response) {
        try{
            $conditions         = $request->getFilterCriteria()->getConditions();
            $cdcliente          = Utilities::getValueInFilterDataSourceByName($conditions,'CDCLIENTE');
            $callRelationship      = Utilities::getValueInFilterDataSourceByName($conditions,'NRSOLISERV');
            $result = $this->callService->verifySlaVisibility($cdcliente, $callRelationship);
            $response->addDataSet(new DataSet('slaDetails', [$result]));
        } catch(\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
    }
}
